/* Copyright (c) 2000-2006 ActiveState Software Inc.
   See the file LICENSE.txt for licensing information. */

/*
 * this file contains general debugging functionality we use
 * during development.  None of these functions should be used
 * in a release build, and this file should not be included in
 * release builds.
 */

var gTimeline = null;

function trace_onload() {
    gTimeline = new trace();
}

function trace() {
    // initialize timeline
    this.timelineService = null;
    if ("@mozilla.org;timeline-service;1" in Components.classes) {
        try {
            this.timelineService =
                Components.classes["@mozilla.org;timeline-service;1"].getService(Components.interfaces.nsITimelineService);
            // and check it is actually operatiing (ie, enabled)
            this.timelineService.indent();
            this.timelineService.outdent();
        }
        catch (ex) { this.timelineService = null }
    }
}

trace.prototype = {
    myPerfTimer: {},
    perfLog: 0,
    perfTimer: function(name, title, reset)
    {
        if (!this.perfLog) {
            this.perfLog = getLoggingMgr().getLogger('perf');
            this.perfLog.setLevel(LOG_DEBUG);
        }
        if (typeof reset == 'undefined' || !reset) {
            reset = false;
        }
        var timenow = new Date();
        if (reset || this.myPerfTimer[name] == 0) this.myPerfTimer[name] = timenow;
        var timediff = timenow.getTime()-this.myPerfTimer[name].getTime();
        //dump("[" + name + ":"+title+"] = "+timediff + " ms\n");
        this.perfLog.info("[" + name + ":"+title+"] = "+timediff + " ms");
        this.myPerfTimer[name] = timenow;
    },
    timestamp: function(name, title)
    {
        if (this.timelineService)
            this.timelineService.mark("perfTimer: " + name + ": " + title)
    },

    _repeatStr: function(str, aCount) {
       var res = "";
       while (--aCount >= 0)
         res += str;
       return res;
    },

    startTimer: function(name)
    {
        if (this.timelineService) {
            this.timelineService.startTimer(name);
        }
    },
    stopTimer: function(name)
    {
        if (this.timelineService) {
            try {
                this.timelineService.stopTimer(name);
            } catch (e) {
                log.warn('Error calling timelineService.stopTimer('+ name + ')');
            }
        }
    },
    resetTimer: function(name)
    {
        if (this.timelineService) {
            try {
                this.timelineService.resetTimer(name);
            } catch (e) {
                log.warn('Error calling timelineService.resetTimer('+ name + ')');
            }
        }
    },
    markTimer: function(name, text)
    {
        if (this.timelineService) {
            if (typeof(text) == 'undefined' || text == '') {
                this.timelineService.markTimer(name);
            } else {
                try {
                    this.timelineService.markTimerWithComment(name, text);
                } catch (e) {
                    this.timelineService.markTimerWithComment(name + '-' + text);
                }
            }
        }
    },
    mark: function(text)
    {
        if (this.timelineService) {
            this.timelineService.mark(text);
        }
    },
    indent: function(text)
    {
        if (this.timelineService) {
            this.timelineService.indent(text);
        }
    },
    outdent: function(text)
    {
        if (this.timelineService) {
            this.timelineService.outdent(text);
        }
    },
    enter: function(text)
    {
        if (this.timelineService) {
            this.timelineService.enter(text);
            this.timelineService.startTimer(text);
        }
    },
    leave: function(text)
    {
        if (this.timelineService) {
            this.timelineService.stopTimer(text);
            this.markTimer(text);
            this.timelineService.resetTimer(text);
            this.timelineService.leave(text);
        }
    }
};

